;=========================================================;
; PciConstants                                   07/28/06 ;
;---------------------------------------------------------;
;                                                         ;
; By Jeff                                                 ;
;=========================================================;

TRUE  EQU 1
FALSE EQU 0

BIT0  EQU 1
BIT1  EQU 2
BIT2  EQU 4
BIT3  EQU 8
BIT4  EQU 10h
BIT5  EQU 20h
BIT6  EQU 40h
BIT7  EQU 80h
BIT8  EQU 100h
BIT9  EQU 200h
BIT10 EQU 400h
BIT11 EQU 800h
BIT12 EQU 1000h
BIT13 EQU 2000h
BIT14 EQU 4000h
BIT15 EQU 8000h
BIT16 EQU 10000h
BIT17 EQU 20000h
BIT18 EQU 40000h
BIT19 EQU 80000h
BIT20 EQU 100000h
BIT21 EQU 200000h
BIT22 EQU 400000h
BIT23 EQU 800000h
BIT24 EQU 1000000h
BIT25 EQU 2000000h
BIT26 EQU 4000000h
BIT27 EQU 8000000h
BIT28 EQU 10000000h
BIT29 EQU 20000000h
BIT30 EQU 40000000h
BIT31 EQU 80000000h




;*****************************************************************************
;* PCI equates (based on Jeff Leyda AC97 wav player SDK :-)
;*****************************************************************************

; PCI equates
; PCI function address (PFA)
; bit 31 = 1
; bit 23:16 = bus number     (0-255)
; bit 15:11 = device number  (0-31)
; bit 10:8 = function number (0-7)
; bit 7:0 = register number  (0-255)

PORTB			EQU	061h
REFRESH_STATUS		EQU	010h	

IO_ADDR_MASK	EQU	0FFFEh		; mask off bit 0 for reading BARs
PCI_INDEX_PORT	EQU	0CF8h
PCI_DATA_PORT	EQU	0CFCh
PCI32		EQU	BIT31		; bitflag to signal 32bit access
PCI16		EQU	BIT30		; bitflag for 16bit access

;PCI32           EQU     BIT30          ; bitflag to signal 32bit access
;PCI16           EQU     BIT29           ; bitflag for 16bit access

PCI_FN0 	EQU	(0 shl 8)
PCI_FN1 	EQU	(1 shl 8)
PCI_FN2 	EQU	(2 shl 8)
PCI_FN3 	EQU	(3 shl 8)
PCI_FN4 	EQU	(4 shl 8)
PCI_FN5 	EQU	(5 shl 8)
PCI_FN6 	EQU	(6 shl 8)
PCI_FN7 	EQU	(7 shl 8)

PCI_CMD_REG	EQU	04h		; reg 04, command reg
IO_ENA		EQU	BIT0		; i/o decode enable
MEM_ENA 	EQU	BIT1		; memory decode enable
BM_ENA		EQU	BIT2		; bus master enable




;*****************************************************************************
;* Intel ICH controller equates (based on Jeff Leyda AC97 wav player SDK :-)
;*****************************************************************************

; supported controllers

INTEL_VID	equ	8086h		; Intel's PCI vendor ID
SIS_VID 	equ	1039h		; SIS PCI vendor ID
NVIDIA_VID	equ	10deh		; NVIDIA vendor ID
AMD_VID 	equ	1022h		; AMD vendor ID

ICH_DID 	equ	2415h		; ICH device ID
ICH0_DID	equ	2425h		; ICH0
ICH2_DID	equ	2445h		; ICH2
ICH3_DID	equ	2485h		; ICH3
ICH4_DID	equ	24c5h		; ICH4
MX440_DID	equ	7195h		; 440MX
SI7012_DID	equ	7012h		; SI7012
NFORCE_DID	equ	01b1h		; NFORCE
AMD8111_DID	equ	764dh		; AMD8111
AMD768_DID	equ	7445h		; AMD768


NAMBAR_REG	equ	10h		; Native Audio Mixer BAR
 NAM_SIZE	equ	256		; 256 bytes required for registers

NABMBAR_REG	equ	14h		; Native Audio Bus Mastering BAR
 NABM_SIZE	equ	64		; 64 bytes required for registers

IRQ_REG 	equ	3ch		; IRQ holder for PCI
INT_REG 	equ	3dh		; INT pin
ICH4_CFG_REG	equ	43h ;42h  ;41h             ; ICH4 config register


; BUS master registers, accessed via NABMBAR+offset
; ICH supports 3 different types of register sets for three types of things :
;
; PCM in (for recording) aka PI
; PCM out (for playback) aka PO
; MIC in (for recording) aka MC

PI_BDBAR_REG		equ	0	; PCM in buffer descriptor BAR
PO_BDBAR_REG		equ	10h	; PCM out buffer descriptor BAR
MC_BDBAR_REG		equ	20h	; MIC in buffer descriptor BAR


; There are 32 entries in the BDL (256 bytes in total)
; Each entry is 8 bytes long. It holds a pointer to a data buffer,
; and the size (in bytes or samples) with execution flags for that buffer.

BDL_SIZE		equ	32*8	; Buffer Descriptor List size
INDEX_MASK		equ	31	; indexes must be 0-31


; 8bit read only
; The Current Index Value is a number of the buffer (0-31) which the codec
; is currently processing.  Once this counter hits 31, it wraps back to 0.

PI_CIV_REG		equ	4	; PCM in Current Index Value (RO)
PO_CIV_REG		equ	14h	; PCM out Current Index Value (RO)
MC_CIV_REG		equ	24h	; MIC in Current Index Value (RO)


; 8bit read/write
; The Last Valid Index is a number of the buffer (0-31) on which the codec
; will stop processing data.

PI_LVI_REG		equ	5	; PCM in Last Valid Index
PO_LVI_REG		equ	15h	; PCM out Last Valid Index
MC_LVI_REG		equ	25h	; MIC in Last Valid Index


; 16bit read/write
; status registers.

PI_SR_REG		equ	6	; PCM in Status Register
PO_SR_REG		equ	16h	; PCM out Status Register
MC_SR_REG		equ	26h	; MIC in Status Register


; status register bitfields

FIFO_ERR		equ	BIT4	; FIFO error (Over/Underrun)  W1TC.
BCIS			equ	BIT3	; Buffer Completion Interrupt Status.
					; Set whenever the last sample in ANY
					; buffer is finished.  Bit is only
					; set when the Interrupt on Complete
					; (BIT4 of control register) is set.
LVBCI			equ	BIT2	; Last Valid Buffer Completion Interrupt
					; Set whenever the codec has processed
					; the last buffer in the buffer list.
					; Will fire an interrupt if IOC bit is
					; set. Probably set after the last
					; sample in the last buffer is
					; processed.  W1TC
CELV			equ	BIT1	; Current Equals Last Valid
					; Bit is RO and remains set until LVI is
					; cleared. Probably set up the start
					; of processing for the last buffer.
DCH			equ	BIT0	; DMA controller halted.
					; Set whenever audio stream is stopped
					; or something else goes wrong.


; 16bit read only
; Position In Current Buffer shows the number of dwords left to be
; processed in the current buffer.

PI_PICB_REG		equ	8	; PCM in position in current buffer(RO)
PO_PICB_REG		equ	18h	; PCM out position in current buffer(RO)
MC_PICB_REG		equ	28h	; MIC in position in current buffer (RO)


; 8bit, read only
; Prefetched Index Value Register.
; Tells which buffer number (0-31) has been prefetched.
; This value probably follows the current index value fairly closely. (CIV+1)

PI_PIV_REG		equ	0ah	; PCM in Prefected index value
PO_PIV_REG		equ	1ah	; PCM out Prefected index value
MC_PIV_REG		equ	2ah	; MIC in Prefected index value


; 8bit
; Control register can only be accessed as an 8bit value.
; Control register.

PI_CR_REG		equ	0bh	; PCM in Control Register
PO_CR_REG		equ	1bh	; PCM out Control Register
MC_CR_REG		equ	2bh	; MIC in Control Register


; control register bitfields.

IOCE			equ	BIT4	; Interrupt On Complete Enable
					; Set this bit if you want an interrupt
					; to fire whenever LVBCI is set.
FEIFE			equ	BIT3	; Fifo Error Interrupt Enable
					; Set if you want an interrupt to fire
					; whenever there is a FIFO (over or
					; under) error.
LVBIE			equ	BIT2	; Last Valid Buffer Interrupt Enable
					; Set if you want an interrupt to fire
					; whenever the completion of the last
					; valid buffer.
RR			equ	BIT1	; Reset busmaster Registers
					; Nukes all regs except bits 4:2
					; of this register.
					; Only set this bit if BIT 0 is 0
RPBM			equ	BIT0	; Start Busmaster operation (Run/Pause)
					; set this bit to start the codec!


; Global Control Register with bitfields

GLOB_CNT_REG		equ	2ch	; Global control register
SMP_24_BIT		equ	BIT23	; 24 bit samples
SMP_20_BIT		equ	BIT22	; 20 bit samples
CHN_PCM_246_MASK	equ	300000h ; 6 channels (not all chips)
CHN_PCM_6		equ	BIT21	; 6 channels (not all chips)
CHN_PCM_4		equ	BIT20	; 4 channels (not all chips)
CHN_PCM_2		equ	0	; 2 channels (stereo)
SEC_RES_EN		equ	BIT5	; Secondary Codec Resume event
					; interrupt enable.
PRI_RES_EN		equ	BIT4	; Primary Codec Resume event
					; interrupt enable.
ACLINK_OFF		equ	BIT3	; Turn off the AC97 link
ACWARM_RESET		equ	BIT2	; Awaken the AC97 link from sleep.
					; registers preserved, bit self clears
ACCOLD_RESET		equ	BIT1	; Reset everything in the AC97 and
					; reset all registers. Not self clearing
GPIIE			equ	BIT0	; GPI Interrupt enable.
					; set if you want an interrupt to
					; fire upon ANY of the bits in the
					; GPI changed?


; Global Status Register with bitfields

GLOB_STS_REG		equ	30h	; Global Status register (RO)
AUDIO_24_BIT		equ	BIT23	; 24 bit sample support
AUDIO_20_BIT		equ	BIT22	; 20 bit sample support
CHN_CAP_6		equ	BIT21	; 6 channels capability
CHN_CAP_4		equ	BIT20	; 4 channels capability
MD3			equ	BIT17	; Modem powerdown status (yawn)
AD3			equ	BIT16	; Audio powerdown status (yawn)
RD_COMPLETE_STS 	equ	BIT15	; Codec read timed out. 0=normal
BIT3SLOT12		equ	BIT14	; shadowed status of bit 3 in slot 12
BIT2SLOT12		equ	BIT13	; shadowed status of bit 2 in slot 12
BIT1SLOT12		equ	BIT12	; shadowed status of bit 1 in slot 12
SEC_RESUME_STS		equ	BIT11	; secondary codec has resumed (and irqed)
PRI_RESUME_STS		equ	BIT10	; primary codec has resumed (and irqed)
SEC_CODEC_RDY		equ	BIT9	; Secondary codec is ready for action
PRI_CODEC_RDY		equ	BIT8	; Primary codec is ready for action
					; software must check these bits before
					; starting the codec!
MIC_IN_IRQ		equ	BIT7	; MIC in caused an interrupt
PCM_OUT_IRQ		equ	BIT6	; One of the PCM out channels IRQed
PCM_IN_IRQ		equ	BIT5	; One of the PCM in channels IRQed
MODEM_OUT_IRQ		equ	BIT2	; Modem out channel IRQed
MODEM_IN_IRQ		equ	BIT1	; Modem in channel IRQed
GPI_STS_CHANGE		equ	BIT0	; Set whenever GPI's have changed.
					; BIT0 of slot 12 also reflects this.


; Codec Write Semaphore Register

ACC_SEMA_REG		equ	34h	; Codec write semaphore register
CODEC_BUSY		equ	BIT0	; codec register I/O is happening
					; self clearing



; Buffer Descriptors List
; Each buffer descriptor list is a set of (up to) 32 descriptors.
; Each descriptor is 8 bytes long. Bytes 0-3 of a descriptor entry point
; to a chunk of memory to either play from or record to. Bytes 4-7 of an
; entry describe various control things detailed below.
; Buffer pointers must always be aligned on a dword boundary.

IOC			equ	BIT31	; Fire an interrupt whenever this
					; buffer is complete.
BUP			equ	BIT30	; Buffer Underrun Policy.
					; If this buffer is the last buffer
					; in a playback, fill the remaining
					; samples with 0 (silence) or not.

; Bits 15:0 contain the length of the buffer, in number of samples,
; (or in number of bytes for the SIS controller) which are 16 bits each,
; coupled in left and right pairs, or 32bits each.
; A value of 0 in these bits means play no samples.




;*****************************************************************************
;* AC97 Codec registers include (based on Jeff Leyda AC97 wav player SDK :-)
;*****************************************************************************

; Not all codecs are created equal. Refer to the spec for your specific codec.
; All registers are 16bits wide.  Access to codec registers over the AC97 link
; is defined by the OEM.
; Secondary codec's are accessed by ORing in BIT7 of all register accesses.


; each codec/mixer register is 16bits

CODEC_RESET_REG 		equ	00	; reset codec
CODEC_MASTER_VOL_REG		equ	02	; master volume
CODEC_HP_VOL_REG		equ	04	; headphone volume
CODEC_MASTER_MONO_VOL_REG	equ	06	; master mono volume
CODEC_MASTER_TONE_REG		equ	08	; master tone (R+L)
CODEC_PCBEEP_VOL_REG		equ	0ah	; PC beep volume
CODEC_PHONE_VOL_REG		equ	0ch	; phone volume
CODEC_MIC_VOL_REG		equ	0eh	; MIC volume
CODEC_LINE_IN_VOL_REG		equ	10h	; line input volume
CODEC_CD_VOL_REG		equ	12h	; CD volume
CODEC_VID_VOL_REG		equ	14h	; video volume
CODEC_AUX_VOL_REG		equ	16h	; aux volume
CODEC_PCM_OUT_REG		equ	18h	; PCM output volume
CODEC_RECORD_SELECT_REG 	equ	1ah	; record select input
CODEC_RECORD_VOL_REG		equ	1ch	; record volume
CODEC_RECORD_MIC_VOL_REG	equ	1eh	; record mic volume
CODEC_GP_REG			equ	20h	; general purpose
CODEC_3D_CONTROL_REG		equ	22h	; 3D control
; 24h is reserved
CODEC_POWER_CTRL_REG		equ	26h	; powerdown control
CODEC_EXT_AUDIO_REG		equ	28h	; extended audio
CODEC_EXT_AUDIO_CTRL_REG	equ	2ah	; extended audio control
CODEC_PCM_FRONT_DACRATE_REG	equ	2ch	; PCM out sample rate
CODEC_PCM_SURND_DACRATE_REG	equ	2eh	; surround sound sample rate
CODEC_PCM_LFE_DACRATE_REG	equ	30h	; LFE sample rate
CODEC_LR_ADCRATE_REG		equ	32h	; PCM in sample rate
CODEC_MIC_ADCRATE_REG		equ	34h	; mic in sample rate


; registers 36-7a are reserved on the ICH

CODEC_VENDORID1_REG		equ	7ch	; codec vendor ID 1
CODEC_VENDORID2_REG		equ	7eh	; codec vendor ID 2


; When 2 codecs are present in the system, use BIT7 to access the 2nd
; set of registers, ie 80h-feh

PRIMARY_CODEC			equ	0	; 0-7F for primary codec
SECONDARY_CODEC 		equ	BIT7	; 80-8f registers for 2ndary

