;=========================================================;
; RmInt                                          07/04/07 ;
;---------------------------------------------------------;
;                                                         ;
; (c) Craig Bamford, All rights reserved.                 ;
;=========================================================;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 
; RealModeInt10h   
;-----------------------------------------------------------------------------
;
;      Action:                   
;          Call realmode int 10h. 
;
;      Input:    
;          AX,BX,CX,DX = same as int 10h function, you want to call.
;
;      Output:             
;          Nothing 
;                               
;      Notes:
;          EAX = bits 32 to 17 = BX, bits 16 to 1 = AX
;          Be carefull may not work will all functions.  
;                     
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
RealModeInt10h:
	pushad
	push  es
	mov   byte[RealModeError],0
	mov   word[RealModeAX],ax
	shr   eax,16
	mov   word[RealModeBX],ax
	mov   word[RealModeCX],cx
	mov   word[RealModeDX],dx
	call  mask_irqs
	mov   al,11h
	out   0x20,al
	out   0xA0,al
	mov   al,0x08
	out   0x21,al
	mov   al,0x70
	out   0xA1,al
	mov   al,4
	out   0x21,al
	mov   al,2
	out   0xA1,al
	mov   al,1
	out   0x21,al
	out   0xA1,al
	jmp   0x20:RealModeInt10h16pm
use16
RealModeInt10h16pm:
	mov   ax,0x28
	mov   ds,ax
	mov   ss,ax
	nop
	mov   bx,[RealModeCS]
	push  bx
	lea   bx,[GoToRmInt10h]
	push  bx
	mov   eax,cr0
	and   al,0xFE
	mov   cr0,eax
	retf
GoToRmInt10h:
	mov ax,cs
	mov ds,ax
	mov ss,ax
	nop
	mov es,ax
	mov fs,ax
	mov gs,ax
	lidt [ridtr]
	push  cs
	pop   ds
	push  ds
	pop   es
	mov  ax,word[RealModeAX]
	mov  bx,word[RealModeBX]
	mov  cx,word[RealModeCX]
	mov  dx,word[RealModeDX]
	int  10h
	jnc  @f
	mov  byte[RealModeError],1
@@:
	mov   word[RealModeAX],ax
	mov   word[RealModeBX],bx
	mov   word[RealModeCX],cx
	mov   word[RealModeDX],dx
	lgdt  [gdtr]
	lidt  [idtr]
	mov   eax,cr0					  
	or    al,1
	mov   cr0,eax
	jmp   sys_code:GoToPmInt10h
use32
GoToPmInt10h:
	mov   ax,sys_data
	mov   ds,ax
	mov   ss,ax
	nop
	mov   es,ax
	mov   es,ax
	mov   gs,ax
	mov   ax,8h
	mov   fs,ax
	call  remap_pic
	call  unmask_irqs
	pop   es
	popad
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 
; RealModeInt1ah
;-----------------------------------------------------------------------------
;
;      Action:                   
;          Call realmode int 1Ah.
;
;      Input:    
;          AX,BX,CX,DX = same as int 10h function, you want to call.
;
;      Output:             
;          Nothing 
;                               
;      Notes:
;          EAX = bits 32 to 17 = BX, bits 16 to 1 = AX
;          Be carefull may not work will all functions.  
;                     
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
RealModeInt1ah:
	pushad
	push  es
	mov   byte[RealModeError],0
	mov   word[RealModeAX],ax
	shr   eax,16
	mov   word[RealModeBX],ax
	mov   word[RealModeCX],cx
	mov   word[RealModeDX],dx
	call  mask_irqs
	mov   al,11h
	out   0x20,al
	out   0xA0,al
	mov   al,0x08
	out   0x21,al
	mov   al,0x70
	out   0xA1,al
	mov   al,4
	out   0x21,al
	mov   al,2
	out   0xA1,al
	mov   al,1
	out   0x21,al
	out   0xA1,al
	jmp   0x20:RealModeInt1Ah16pm
use16
RealModeInt1Ah16pm:
	mov   ax,0x28
	mov   ds,ax
	mov   ss,ax
	nop
	mov   bx,[RealModeCS]
	push  bx
	lea   bx,[GoToRmInt1Ah]
	push  bx
	mov   eax,cr0
	and   al,0xFE
	mov   cr0,eax
	retf
GoToRmInt1Ah:
	mov ax,cs
	mov ds,ax
	mov ss,ax
	nop
	mov es,ax
	mov fs,ax
	mov gs,ax
	lidt [ridtr]
	push  cs
	pop   ds
	push  ds
	pop   es
	mov  ax,word[RealModeAX]
	mov  bx,word[RealModeBX]
	mov  cx,word[RealModeCX]
	mov  dx,word[RealModeDX] 
	int  1ah
	jnc  @f
	mov  byte[RealModeError],1
@@:
	mov   word[RealModeAX],ax
	mov   word[RealModeBX],bx
	mov   word[RealModeCX],cx
	mov   word[RealModeDX],dx
	lgdt  [gdtr]
	lidt  [idtr]
	mov   eax,cr0					  
	or    al,1
	mov   cr0,eax
	jmp   sys_code:GoToPmInt1Ah
use32
GoToPmInt1Ah:
	mov   ax,sys_data
	mov   ds,ax
	mov   ss,ax
	nop
	mov   es,ax
	mov   fs,ax
	mov   gs,ax
	mov   ax,8h
	mov   fs,ax
	call  remap_pic
	call  unmask_irqs
	pop   es
	popad
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 
; SetDex4uFonts                            
;-----------------------------------------------------------------------------
;
;      Action:                   
;          Goes to realmode and back sets 50x80 mode/fonts 
;
;      Input:    
;          Nothing
;
;      Output:
;          Nothing
;                          
;      Notes: Call this if you have set anyother mode to return back to 
;             Standard DexOS text mode.
;                     
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;SetDex4uFonts:
SetDex4uFontsInt:
	pushad
	push  es
	call  mask_irqs
	jmp   0x20:SetDex4uFonts16PM
use16
SetDex4uFonts16PM:
	mov   ax,0x28
	mov   ds,ax
	mov   ss,ax
	nop
	mov   bx,[RealModeCS]
	push  bx
	lea   bx,[GoToRmSetDex4uFonts]
	push  bx
	mov   eax,cr0
	and   al,0xFE
	mov   cr0,eax
	retf
GoToRmSetDex4uFonts:
	mov ax,cs
	mov ds,ax
	mov ss,ax
	nop
	mov es,ax
	mov fs,ax
	mov gs,ax
	lidt [ridtr]
	push  cs
	pop   ds
	push  ds
	pop   es
	mov  ax,0x0003
	int  10h
	call  TextReSize
	lgdt  [gdtr]
	lidt  [idtr]
	mov   eax,cr0					  
	or    al,1
	mov   cr0,eax
	jmp   sys_code:GoToPmSetDex4uFonts
use32
GoToPmSetDex4uFonts:
	mov   ax,sys_data
	mov   ds,ax
	mov   ss,ax
	nop
	mov   es,ax
	mov   fs,ax
	mov   gs,ax
	mov   ax,8h
	mov   fs,ax
	call  unmask_irqs
	pop   es
	popad
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; VesaRmInt
;-----------------------------------------------------------------------------
;
;      Action:
;          Sets vesa mode
;
;      Input:
;          CX = mode number
;
;      Output:
;          set's CF to 1, on error
;
;      Notes:
;          Need vesa info structure filling in afer calling this, use  function  31
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
VesaRmInt:
	pushad
	push  es
	mov   word[RealModeCX],cx
@@:
	mov   byte[vesaRmError],0
	call  mask_irqs
	jmp   0x20:VesaRmInt16PM
use16
VesaRmInt16PM:
	mov   ax,0x28
	mov   ds,ax
	mov   ss,ax
	nop
	mov   bx,[RealModeCS]
	push  bx
	lea   bx,[VesaRmInt16RM]
	push  bx
	mov   eax,cr0
	and   al,0xFE
	mov   cr0,eax
	retf
VesaRmInt16RM:
	mov   ax,cs
	mov   ds,ax
	mov   ss,ax
	nop
	mov   es,ax
	mov   fs,ax
	mov   gs,ax
	lidt  [ridtr]
	push  cs
	pop   ds
	push  ds
	pop   es
	mov byte[vesaRmError],0
	mov  cx,word[RealModeCX]
	mov   ax,4f01h
	mov   di,Mode_Info	
	and   cx,0xfff
	int   10h 
	cmp   [VESAInfo_Version],0x200
	jb    NoVesaMode
	mov   ax,[ModeInfo_ModeAttributes]
	and   al,00000001b
	cmp   al,1
	jne   NoVesaMode
	mov   ax,[ModeInfo_ModeAttributes]
	and   al,10000000b
	cmp   al,10000000b
	jne   NoVesaMode
	mov   byte[NoVesa],0
	mov   ax,0x4f02
	mov   bx,word[RealModeCX]
	int   10h
	cmp   ax,004Fh
	je    VesaOK
NoVesaMode:
	mov byte[vesaRmError],1
VesaOK:
	lgdt  [gdtr]
	lidt  [idtr]
	mov   eax,cr0					  
	or    al,1
	mov   cr0,eax
	jmp   sys_code:GoToPmVesaRmInt
use32
GoToPmVesaRmInt:
	mov   ax,sys_data
	mov   ds,ax
	mov   ss,ax
	nop
	mov   es,ax
	mov   fs,ax
	mov   gs,ax
	mov   ax,8h
	mov   fs,ax
	call  unmask_irqs
	pop   es
	popad
	mov   ah,0
	cmp   byte[vesaRmError],0
	je    @f
	mov   ah,1
@@:
	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 
; RealModeRegs   
;-----------------------------------------------------------------------------
;
;      Action:                   
;          Fills in AX,BX,CX,DX with result from  function 3
;
;      Input:    
;          Nothing
;
;      Output:             
;          AX,BX,CX,DX = same as returned by int 10h function, you want to call
;          Set's CF to 1 on error. 
;                    
;      Notes:
;                     
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
RealModeRegs:
	mov  ax,word[RealModeAX]
	mov  bx,word[RealModeBX]
	mov  cx,word[RealModeCX]
	mov  dx,word[RealModeDX] 
	cmp  [RealModeError],0 
	je   @f
	stc
	ret
@@:
	clc
	ret

 ;----------------------------------------------------;
 ; RealmodeInt13h                                     ;
 ;----------------------------------------------------;
RealmodeInt13h:
	pushad
	push  es
	mov   byte[RealModeError],0
	mov   word[RealModeAX],ax
	mov   word[RealModeBX],bx
	mov   word[RealModeCX],cx
	mov   word[RealModeDX],dx
	call  mask_irqs
	mov   al,11h
	out   0x20,al
	out   0xA0,al
	mov   al,0x08
	out   0x21,al
	mov   al,0x70
	out   0xA1,al
	mov   al,4
	out   0x21,al
	mov   al,2
	out   0xA1,al
	mov   al,1
	out   0x21,al
	out   0xA1,al
	jmp   0x20:RealmodeInt13h16PM
use16
RealmodeInt13h16PM:
	mov   ax,0x28
	mov   ds,ax
	mov   ss,ax
	nop
 ;----------------------------------------------------;
 ; push real-mode CS:IP.                              ;
 ;----------------------------------------------------;  
	mov   bx,[RealModeCS]
	push  bx
	lea   bx,[RealmodeInt13h16RM]
	push  bx
 ;----------------------------------------------------;
 ; clear PE [pmode enable] bit and return to Rmode.   ;
 ;----------------------------------------------------; 
	mov   eax,cr0
	and   al,0xFE
	mov   cr0,eax
	retf
 ;----------------------------------------------------;
 ;  16-bit real mode again                            ;
 ;----------------------------------------------------;                                   
RealmodeInt13h16RM:
	mov   ax,cs
	mov   ds,ax
	mov   ss,ax
	nop
	mov   es,ax
	mov   fs,ax
	mov   gs,ax
 ;----------------------------------------------------;
 ; point to real-mode IDTR.                           ;
 ;----------------------------------------------------; 
	lidt  [ridtr]
	push  cs
	pop   ds
	push  ds
	pop   es
 ;----------------------------------------------------;
 ;  Do real mode interrupts                           ;
 ;----------------------------------------------------;
	mov   ax,word[RealModeAX]
	mov   bx,word[RealModeBX]
	mov   cx,word[RealModeCX]
	mov   dx,word[RealModeDX]   
	mov   bx,OffsetDma2Buffer2
	int   13h
	jnc   @f
	mov   byte[RealModeError],1
@@:
	mov   word[RealModeAX],ax
	mov   word[RealModeBX],bx
	mov   word[RealModeCX],cx
	mov   word[RealModeDX],dx
 ;----------------------------------------------------;
 ;  Lets go back to pmode                             ;
 ;----------------------------------------------------;      
	lgdt  [gdtr]					  
	lidt  [idtr]
	mov   eax,cr0					  
	or    al,1
	mov   cr0,eax
 ;----------------------------------------------------;
 ;  jumps to do_pm                                    ;
 ;----------------------------------------------------;
	jmp   sys_code:GoPmRealmodeInt13h
 ;----------------------------------------------------;
 ;  32-bit protected mode                             ;
 ;----------------------------------------------------;
use32
GoPmRealmodeInt13h:
	mov   ax,sys_data
	mov   ds,ax
	mov   ss,ax
	nop
	mov   es,ax
	mov   fs,ax
	mov   gs,ax
	mov   ax,8h
	mov   fs,ax
	call  remap_pic
	call  unmask_irqs
	cmp   [RealModeError],1
	je    Int13hBIOSError
Int13hBIOSSuccsess:  
	pop   es
	popad					     
	mov   ax,word[RealModeAX]
	mov   esi,OffsetDma2Buffer2
	clc
	ret
Int13hBIOSError:  
	pop   es
	popad	     
	mov  ax,word[RealModeAX]
	stc
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RebootSystem
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
RebootSystem:
       call  mask_irqs
       jmp   0x20:do_Reboot
use16
do_Reboot:  
       mov   ax,0x28
       mov   ds,ax
       mov   ss,ax
       nop
 ;----------------------------------------------------;
 ; push real-mode CS:IP.                              ;
 ;----------------------------------------------------;  
       mov   bx,[RealModeCS]
       push  bx
       lea   bx,[do_ReallyMode]
       push  bx
 ;----------------------------------------------------;
 ; clear PE [pmode enable] bit and return to Rmode.   ;
 ;----------------------------------------------------; 
       mov   eax,cr0
       and   al,0xFE
       mov   cr0,eax
       retf
 ;----------------------------------------------------;
 ;  16-bit real mode again                            ;
 ;----------------------------------------------------;                                   
do_ReallyMode:	
       mov ax,cs
       mov ds,ax
       mov ss,ax
       nop
       mov es,ax
       mov fs,ax
       mov gs,ax
 ;----------------------------------------------------;
 ; point to real-mode IDTR.                           ;
 ;----------------------------------------------------; 
       lidt [ridtr]
       push  cs
       pop   ds
       push  ds
       pop   es
ReBootMe:
       DB 0EAh					      ; Machine language to jump to
						      ;    address FFFF:0000 (reboot)
       DW 0000h
       DW 0FFFFh				      ; No return required
						      ;    we're rebooting!

       jmp $					      ; Just incase ;-)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ExitDOS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
use32	     
ExitDOS:
	call  mask_irqs
	mov   al,11h
	out   020h,al
	out   0A0h,al
	mov   al,08h
	out   021h,al
	mov   al,070h
	out   0A1h,al
	mov   al,4
	out   021h,al
	mov   al,2
	out   0A1h,al
	mov   al,1
	out   021h,al
	out   0A1h,al
	jmp   0x20:dos_Reset
    
use16
dos_Reset:  

	mov   ax,0x28
	mov   ds,ax
	mov   ss,ax
	nop
 ;----------------------------------------------------;
 ; push real-mode CS:IP.                              ;
 ;----------------------------------------------------;  
	mov   bx,[RealModeCS]
	push  bx
	lea   bx,[do_DOS]
	push  bx
 ;----------------------------------------------------;
 ; clear PE [pmode enable] bit and return to Rmode.   ;
 ;----------------------------------------------------; 
	mov   eax,cr0
	and   al,0xFE
	mov   cr0,eax
	retf
 ;----------------------------------------------------;
 ;  16-bit real mode again                            ;
 ;----------------------------------------------------;                                   
do_DOS:
	mov   ax,cs
	mov   ds,ax
	mov   ss,ax
	nop
	mov   es,ax
	mov   fs,ax
	mov   gs,ax
 ;----------------------------------------------------;
 ; point to real-mode IDTR.                           ;
 ;----------------------------------------------------; 
	lidt [ridtr]
	push  cs
	pop   ds
	push  ds
	pop   es
	mov   al,0
	out   0A1h,al
	out   021h,al
	sti
	mov   ax,0x0003
	Int   10h
	cmp   byte [DosLoaded],1
	je    BackToDos
	call  SafeToTurnOffMessage
BackToDos:
	mov ah,4Ch
	int 21h
	jmp $
use32
 ;----------------------------------------------------;
 ;  Data                                              ;
 ;----------------------------------------------------; 
RealModeError  db 0 
vesaRmError    db 0
RealModeAX     dw 0
RealModeBX     dw 0
RealModeCX     dw 0
RealModeDX     dw 0
BiosIntNumber  dw 0
