;=========================================================;
; TextMode50                                     07/28/06 ;
;---------------------------------------------------------;
;                                                         ;
; (c) Craig Bamford, All rights reserved.                 ;
;=========================================================;

 ;----------------------------------------------------;
 ; gets the text pos                                  ;
 ;----------------------------------------------------;
get_text_pos:
	mov   al,[screen_x]
	mov   ah,[screen_y]
	ret

 ;----------------------------------------------------;
 ; Sets the text pos                                  ;
 ;----------------------------------------------------;
set_text_pos:
	mov   [screen_x],al
	mov   [screen_y],ah
	ret

 ;----------------------------------------------------;
 ; TextColor                        ; Sets text color ;
 ;----------------------------------------------------;
TextColor:
	mov   byte[textcolor],al
	ret

 ;----------------------------------------------------;
 ; SetCursorPos                     ; Sets cursor pos ;
 ;----------------------------------------------------;
SetCursorPos:
	cmp   al,80
	jnb   @f
	cmp   ah,50
	jnb   @f
	call  set_text_pos 
	call  set_cursor_pos
	mov   ah,0
	ret
@@:
	stc
	ret

 ;'''''''''''''''''''''''''''''''''''''''''''''''''''';
 ; Prints char               ; print's a char from al ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;  Input:                                            ;
 ;      Al Ascii character to write.                  ;
 ;                                                    ;
 ; Output:                                            ;
 ;      None.                                         ;
 ;....................................................;
print_char:
	pushad					 
	push  es
	push  ds
	cli
	push   ax
	mov   ax,8h			     
	mov   es,ax			     
	pop   ax	
	mov   edi,0xb8000
	call  get_scr_pointer
	mov   ah,[textcolor]
	mov   [es:edi],ax		     
	call  inc_scr_pointer
	sti
	pop   ds
	pop   es
	popad
	ret

 ;'''''''''''''''''''''''''''''''''''''''''''''''''''';
 ; Prints char cursor pos    ; print's a char from al ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;  Input:                                            ;
 ;      Al Ascii character to write.                  ;
 ;                                                    ;
 ; Output:                                            ;
 ;      None.                                         ;
 ; NOTES:                                             ;
 ;       Prints char from al, also moves cursor.      ;
 ;....................................................;
PrintCharCursorPos:
	pushad					 
	push  es
	push  ds
	cli
	push  ax
	mov   ax,8h			     
	mov   es,ax			     
	pop   ax	
	mov   edi,0xb8000
	call  get_scr_pointer
	mov   ah,[textcolor]
	mov   [es:edi],ax		     
	call  inc_scr_pointer
	sti
	call  set_cursor_pos
	pop   ds
	pop   es
	popad
	ret


 ;'''''''''''''''''''''''''''''''''''''''''''''''''''';
 ; Prints string      ; does what it say's on the box ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;  Input:                                            ;
 ;      es:esi points to asciiz string to write.      ;
 ;                                                    ;
 ; Output:                                            ;
 ;      None.                                         ;
 ;....................................................;
print_string:
	pushad					 
	push  es
	push  ds
aloop:
	mov   al,[esi]
	cmp   al,0					   ; 0x00 = end of string
	je    gend
	cmp   al,0xd2					   ; 0xd2 = next byte is color byte
	je    ColorChange
	cmp   al,0x0d					   ; 0x0d = CR ( = \n )
	jne   no_cr
	call  carriage_return
	jmp   a1
ColorChange:
	inc   esi
	mov   al,[esi]
	mov   byte[textcolor],al
	inc   esi
	jmp   aloop
no_cr:
	call  print_char
a1:	inc   esi
	jmp   aloop
gend:
	call  set_cursor_pos
	pop   ds
	pop   es
	popad
	ret

 ;----------------------------------------------------;
 ; write hex 32        ;prints a hex number from eax. ;
 ;----------------------------------------------------;
write_hex32:
	pushad
	mov   ebx,eax
	mov   ecx,4
loop1:
	rol   eax,8					   
	call  write_hex8
	loop   loop1
	popad
	ret

 ;----------------------------------------------------;
 ; write hex16                                        ;
 ;----------------------------------------------------;
write_hex16:
	pushad
	mov   ebx,eax
	shr   eax,8
	call  write_hex8
	mov   eax,ebx
	call  write_hex8
	popad
	ret


 ;----------------------------------------------------;
 ; write hex                                          ;
 ;----------------------------------------------------;
write_hex8:
	push  eax
	push  eax
	shr   al,4					   
	call  hexget					  
	call  print_char
	pop   eax
	call  hexget					   
	call  print_char
	pop   eax
	ret

 ;----------------------------------------------------;
 ; Carriage return                       ; next line  ;
 ;----------------------------------------------------;
carriage_return:
	push  eax
	mov   [screen_x],0
	mov   ah,[screen_y]	
	cmp   ah,49		     
	jb    no_scroll
	call  scroll
	jmp   cend
no_scroll:
	inc   ah    
	mov   [screen_y],ah	
cend:	pop   eax
	ret

 ;----------------------------------------------------;
 ; Scroll                      ; scroll's screen up 1 ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;   Input:                                           ;
 ;          none.                                     ;
 ;  Output:                                           ;
 ;          none.                                     ;
 ;....................................................;
scroll:
	pushad
	push  fs
	push  es
	push  ds
	mov   ax,8h
	mov   fs,ax
	mov   al,0
	mov   ah,49	      
	call  set_text_pos
	mov   edi,0xb8000
	add   edi,80*2*4 
	mov   esi,0xb8000
	add   esi,80*2*5 
	mov   ecx,80*46*2/4 
	cli
mloop:	
	mov   eax,[fs:esi]
	add   esi,4
	mov   [fs:edi],eax
	add   edi,4
	dec   ecx
	jnz   mloop
	sti
	call  set_cursor_pos
	pop   ds
	pop   es
	pop   fs
	popad
	ret

 ;----------------------------------------------------;
 ; cls_text                     ; clear's full screen ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;   Input:                                           ;
 ;          none.                                     ;
 ;  Output:                                           ;
 ;          none.                                     ;
 ;....................................................;
cls_text:
	push  eax
	push  ecx
	push  esi
	push  edi
	push  fs
	mov   ax,8h
	mov   fs,ax
	cli
	mov   ecx,80*50     
	mov   edi,0B8000h
	mov   al," "		
	mov   bl,0Fh		
cls_loop:
	mov   byte [fs:edi],al
	inc   edi
	mov   byte [fs:edi],bl
	inc   edi
	dec   ecx
	jnz   cls_loop
	mov   [screen_x],byte 0
	mov   [screen_y],byte 4 	
	sti
	pop   fs
	pop   edi
	pop   esi
	pop   ecx
	pop   eax
	ret


 ;----------------------------------------------------;
 ; clstext                      ; clear's text screen ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;   Input:                                           ;
 ;          none.                                     ;
 ;  Output:                                           ;
 ;          none.                                     ;
 ;....................................................;
clstext:
	push  eax
	push  ecx
	push  esi
	push  edi
	push  fs
	mov   ax,8h
	mov   fs,ax
	cli
	mov   ecx,80*46 
	mov   edi,0B8000h
	add   edi,80 * 4* 2 
	mov   al," "		
	mov   bl,0x0f		
cls_loop1:
	mov   byte [fs:edi],al
	inc   edi
	mov   byte [fs:edi],bl
	inc   edi
	dec   ecx
	jnz   cls_loop1
	mov   [screen_x],byte 0
	mov   [screen_y],byte 4 
	sti
	call  set_cursor_pos ;craig
	pop   fs
	pop   edi
	pop   esi
	pop   ecx
	pop   eax
	ret

 ;----------------------------------------------------;
 ; Inc screen pointer                                 ;
 ;----------------------------------------------------;
inc_scr_pointer:
	push  eax
	cld
	mov   al,[screen_x]
	cmp   al,79		     
	jb    no_incy
	call  carriage_return
	jmp   dend
no_incy:
	inc   al
	mov   [screen_x],al
dend:	pop   eax
	ret

 ;----------------------------------------------------;
 ; Gets screen pointer                                ;
 ;----------------------------------------------------;
get_scr_pointer:
	push  eax					   
	push  ebx
	xor   ebx,ebx
	mov   bl,[screen_x]
	shl   ebx,1
	add   edi, ebx
	mov   bl,[screen_y]
	mov   eax,0xa0
	mul   bx
	add   edi,eax
	pop   ebx
	pop   eax
	ret

 ;'''''''''''''''''''''''''''''''''''''''''''''''''''';
 ; Sets cursor pos                                    ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;  Input:                                            ;
 ;     [screen_x]    points to X                      ;
 ;                                                    ;
 ;     [screen_Y]    points to Y                      ;
 ;                                                    ;
 ; Output:                                            ;
 ;      None.                                         ;
 ;....................................................;
set_cursor_pos:
	push  eax
	push  ebx
	push  ecx
	push  edx
	xor   ebx,ebx
	mov   bl,[screen_x]				     
	mov   ecx,ebx
	mov   bl,[screen_y]				      
	mov   eax,80
	mul   bx
	add   eax,ecx					  
	mov   edx,0x3d4 
	mov   ecx,eax
	mov   al,0x0f
	out   dx,al
	mov   eax,ecx
	inc   edx
	out   dx,al
	mov   al,0x0e
	dec   edx
	out   dx,al
	mov   eax,ecx
	mov   al,ah
	inc   edx
	out   dx,al					     

	pop   edx
	pop   ecx
	pop   ebx
	pop   eax
	ret

 ;----------------------------------------------------;
 ; Compare_String                                     ;
 ;                                                    ;
 ; NOTE: I left this in as a joke,                    ;
 ; So no comments on compear                          ;
 ;----------------------------------------------------;
Compear_String:
	pushad
	push  es
	mov   ax,sys_data				    
	mov   es,ax					    
	mov   edi,TmpFileName1
	mov   esi,CommandBuffer 			    
	add   esi,4
	xor   cx,cx
	mov   cl,byte[CdNameLen]       
FindNameCycle:
	cmp   byte [es:edi],ch
	je    FindNameFailed	      
	pusha
	repe  cmpsb
	popa
	je    FindNameFound
FindNameFailed:
	stc
FindNameFound:
	pop   es
	popad
	ret

 
 ;'''''''''''''''''''''''''''''''''''''''''''''''''''';
 ; CompearString                                      ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;  Input:                                            ;
 ;     es:edi = string1 buffer (0 termanated)         ;
 ;     ds:esi = string2 buffer (0 termanated)         ;
 ;        ecx = string size                           ;
 ;                                                    ;
 ; Output:                                            ;
 ;      set CF1 on error (jc on error)                ;
 ;....................................................;
 ; NOTE: I left this in as a joke,                    ;
 ; So no comments on compear                          ;
 ;----------------------------------------------------;
CompearString:
	pushad
	push  es
	push  ds
	mov   ax,18h
	mov   ds,ax				     
	mov   es,ax
	cmp   ecx,0
	je    CmpNameFailed
	cld					
	repe  cmpsb
	jne   CmpNameFailed
	jmp   CmpNameFound 
CmpNameFailed:
	pop   ds
	pop   es
	popad
	stc
	ret

CmpNameFound:
	pop   ds
	pop   es
	popad
	clc
	ret


 ;----------------------------------------------------;
 ; hexget         convers from ASCII to hexadecimal.  ;
 ;----------------------------------------------------;
hexget:
	and   eax,0x0000000f
	or    eax,0x00000030
	cmp   eax,0x39
	ja    add7
	ret
add7:	add   eax,7		
	ret

 ;----------------------------------------------------;
 ; UpperCase.       ; covverts a string to uppercase. ;
 ;----------------------------------------------------;
 ;                                                    ;
 ; es:edi = string buffer (0 termanated)              ;
 ;----------------------------------------------------;
UpperCase:
	pushad
	push  es
	mov   ax,sys_data				
	mov   es,ax    
UcaseNextChar:
	mov   al,byte[es:edi]
	cmp   al,0
	je    UcaseDone
	cmp   al,0x61
	jb    DontUcaseChar
	cmp   al,0x7a
	ja    DontUcaseChar
	sub   al,0x20
	mov   byte[es:edi],al  
DontUcaseChar:
	inc   edi
	jmp   UcaseNextChar
UcaseDone:
	pop   es
	popad
	ret


 ;----------------------------------------------------;
 ; GetCommandUser                                     ;
 ;----------------------------------------------------;
GetCommandUser:
	pushad
	push  es
	cmp   bx,100
	je    @f
	mov   dx,CmdSizeUser 
@@:    
	push  es
	pushad 
	mov   ax,18h				    
	mov   es,ax	
	mov   edi,CommandBufferUser
	mov   cx,128
	mov   al,' '
	rep   stosb 
	popad	
	pop   es			  
	mov   edi,CommandBufferUser
	call  GetStringUser
	jc    EscPressed
	pop   es
	popad
	mov   edi,CommandBufferUser
	mov   cx,word[UserCount]
	clc
	ret
EscPressed:
	pop   es
	popad
	stc 
	ret

 ;----------------------------------------------------;
 ; GetStringUser                                      ;
 ;----------------------------------------------------;
GetStringUser:
	pushad
	push  es
	mov   ax,sys_data				 
	mov   es,ax    
	mov   edi,CommandBufferUser
	mov   ecx,0
	cmp   dx,1					 
	jbe   EmptyBufferUser 
	dec   dx					 
@@:
	call  WaitForKeyPress
	cmp   [keybuffer],0
	je    @b
	mov   al,[keybuffer]
	mov   [keybuffer],0
	cmp   al,13					 
	jz    ExitUser
	cmp   al,27					 
	jz    ExitEsc
	cmp   al, 8					 
	jne   AddToBufferUser
	jcxz  @b					 
	cmp   [screen_x],0
	je    @b
	dec   cx
	dec   edi
	dec   [screen_x]
	mov   al,' '
	call  print_char
	dec   [screen_x]
	call  set_cursor_pos
	jmp   @b
AddToBufferUser:
	cmp   cx,dx					 
	jae   @b					  
	stosb
	inc   cx
	call  PrintCharCursorPos
	jmp   @b
ExitUser:
	mov  byte [es:edi],0				 
EmptyBufferUser:
	mov [UserCount],cx
	clc
	pop   es
	popad
	ret
ExitEsc:
	stc 
	pop   es
	popad
	ret

;====================================================;
; Get Command NEW.                                   ;
;====================================================;
GetCommandNEW:
	mov   dx,32 ;CmdSizeNEW
	mov   edi,CommandBufferNEW ;CommandBuffer
	call  GetStringNEW
	jc    GetCommandNEWesc
	mov   edi,CommandBuffer2NEW
	mov   cx,word[CliBuffcountNEW]
	clc
	ret

GetCommandNEWesc:
	stc
	ret

;=========================================================;
; Get String NEW                                 11/12/06 ;
;---------------------------------------------------------;
; Get string from user, the received string is            ;
; written to buffer                                       ;
; at    ES:EDI, buffer size should be in DX.              ;
;                                                         ;                                                             
;=========================================================;
GetStringNEW:
	pushad
	push  es
	mov   ax,sys_data				 
	mov   es,ax    
	mov   edi,CommandBufferNEW
	mov   ecx,0
	cmp   dx,1					 
	jbe   EmptyBufferNEW
	dec   dx					 
@@:
	call  WaitForKeyPress
	mov   al,[keybuffer]
	mov   ah,[ScanCode]
	mov   [ScanCode],0
	mov   [keybuffer],0
	cmp   al,13					 
	jz    ExitNEW
	cmp   al,27
	jz    ExitEscGetStringNEW
	cmp   ah,0x4d
	jne   La1NEW
	cmp   [CliBuffcountNEW],0
	jbe   @b
	cmp   cx,[CliBuffcountNEW]
	jae    @b
	mov   esi,CommandBuffer2NEW
	push  eax
	xor   eax,eax
	add   ax,cx
	add   esi,eax
	pop   eax
	lodsb
	jmp   AddToBufferNEW
La1NEW:
	cmp   ah,0x4b
	je    La2NEW
	cmp   al, 0
	je    @b
	cmp   al, 8					 
	jne    AddToBufferNEW
La2NEW:
	jcxz  @b					 
	cmp   [screen_x],20
	je    @b
	dec   cx
	dec   edi
	dec   [screen_x]
	mov   al,' '
	call  print_char
	dec   [screen_x]
	call  set_cursor_pos
	jmp   @b
AddToBufferNEW:
	cmp   cx,dx					 
	jae   @b
AddToBuffer1NEW:
	stosb
	inc   cx
	call  PrintCharCursorPos
	jmp   @b
ExitNEW:
	mov   byte [es:edi],0				  
EmptyBufferNEW:
	mov   [craicountNEW],cx
	mov   [CliBuffcountNEW],cx
	mov   edi,CommandBuffer2NEW
	mov   esi,CommandBufferNEW
	mov   cx,128
	rep   movsb
	pop   es
	popad
	clc
	ret

ExitEscGetStringNEW:
	pop   es
	popad
	stc
	ret


 ;----------------------------------------------------;
 ; Data.                                              ;
 ;----------------------------------------------------;
UserCount	   dw 0
CmdSizeUser =	   0x80
CliBuffcountNEW    dw 0
craicountNEW	   dw 0
CommandBufferUser: times   128	db 0
CommandBufferNEW:  times   128	db 0
CommandBuffer2NEW: times   128	db 0
