
 ;----------------------------------------------------;
 ;  unhandled int.                                    ;
 ;----------------------------------------------------;
unhandled_int:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809E], "U"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ; unhandled_irq1                                     ;
 ;----------------------------------------------------;
unhandled_irq1:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "&"
	mov   al,0x20
	out   0xa0,al
	out   0x20,al
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  nmi interrupt.                                    ;
 ;----------------------------------------------------;
nmi_interrupt:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809E], "N"
reboot1:
	jmp   reboot1
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  clock tick.                                       ;
 ;----------------------------------------------------;
clock_tick:
	pushad
	push  es
	push  ds
TimerCode:
	test  [TimerOn],-1				; See if timer on
	jz    NoTimer					; no jump lable
	dec   [Timer]					; Yes, decrement
	jnz   NoTimer					; Not done
	and   [TimerOn],0				; finished, inform task
NoTimer:
	test  [TimerOnNw],-1				; See if floppy motor timer is on.
	jz    NoFddTimer				; no jump lable
	dec   [TimerNw] 				; Yes, decrement
	jnz   NoFddTimer				; Not done
	and   [TimerOnNw],0				; mov floppy time switch off
NoFddTimer:
	test  [TimerProgOn],-1				; See if program timer is on.
	jz    NoProgTimer				; no jump lable
	inc   [TimerProg]				; Yes, increas
	cmp   [TimerProg],0x00ffffff
	jb    NoProgTimer				; Not done
	and   [TimerProgOn],0				; Prog time switch off
NoProgTimer:
	call  irq_clear
	mov   [Tick18],1
	inc   [NumberOfTicks]
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  keyboard irq.                                     ;
 ;----------------------------------------------------;
keyboard_irq:
	pushad
	push  es
	push  ds
       ;mov   byte [fs:0xB809C], "K"
	cmp   [GameKeyBoardOn],1
	jne   NoGameKeyBoard
	call  GamesKeyBoard
	jmp   ClearIRQgo
NoGameKeyBoard:
	call  key_board
ClearIRQgo: 
	mov   ah,[ScanCode2]
	mov   [ScanCode2],0
	call  irq_clear
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  Mouse irq.                                        ;
 ;----------------------------------------------------;
Mouse_irq:
	pushad
	push  es
	push  ds
	call  MouseMovmentClick
	mov   al,0x20
	out   0xa0,al
	out   0x20,al
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  floppy irq.                                       ;
 ;----------------------------------------------------;
floppy_irq:
	pushad
	push  es
	push  ds
	or    [done],1
	call  irq_clear
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  page_fault                                        ;
 ;----------------------------------------------------; 
page_fault:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "P"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  div_error                                         ;
 ;----------------------------------------------------;
div_error:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "D"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  debug_exception                                   ;
 ;----------------------------------------------------;
debug_exception:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "d"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  int3_trap                                         ;
 ;----------------------------------------------------;
int3_trap:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "3"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  into_trap                                         ;
 ;----------------------------------------------------;
into_trap:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "O"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  bound_trap                                        ;
 ;----------------------------------------------------;
bound_trap:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "B"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  invalid_instruction                               ;
 ;----------------------------------------------------;
invalid_instruction:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "I"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  no_coprocessor                                    ;
 ;----------------------------------------------------;
no_coprocessor:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "n"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  double_fault                                      ;
 ;----------------------------------------------------;
double_fault:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "8"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  coprocessor_segment_overrun                       ;
 ;----------------------------------------------------;
coprocessor_segment_overrun:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "9"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  invalid_tss                                       ;
 ;----------------------------------------------------;
invalid_tss:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "A"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  segment_not_present                               ;
 ;----------------------------------------------------;
segment_not_present:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "B"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  stack_fault                                       ;
 ;----------------------------------------------------;
stack_fault:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "S"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  gpf                                               ;
 ;----------------------------------------------------;
gpf:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "D"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  coprocessor_error                                 ;
 ;----------------------------------------------------;
coprocessor_error:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "e"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  alignment_check                                   ;
 ;----------------------------------------------------;
alignment_check:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "F"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  machine_check                                     ;
 ;----------------------------------------------------;
machine_check:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "G"
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  int_40h                         ( int functions)  ;
 ;----------------------------------------------------;
int_40h:
	pushad
	push  es
	push  ds
	cmp   ah,0ah					; Test for 0ah = "Load CallTable Address"
	jne   ErrorInt40h
LoadCallTableAddress:
	mov   esi,CallTable
	mov   ecx,256*2
	cmp   al,0
	je    FillTable
	mov   ecx,1
FillTable:
	cld
	rep   movsd
sucsess40h:
	pop   ds					; universal return procedure
	pop   es
	popad
	and	byte [esp+8],not 1			; clear CF
	iret

ErrorInt40h:
	pop   ds					; universal return error procedure
	pop   es
	popad
	or	byte [esp+8],1				; set CF
	iret

 ;----------------------------------------------------;
 ;  int_41h                         ( int functions)  ;
 ;                                                    ;
 ; You can call any function in the calltable         ;
 ; by puting the offset number in ebx and calling int ;
 ; 41h                                                ;
 ; So to call the first function in the calltable     ;
 ; you would put 1 in ebx and fill the rest ogf the   ;
 ; Regs as normal.                                    ;
 ;                                                    ;
 ;----------------------------------------------------;
int_41h:
	cmp   ebx,0
	jne   NotZero
	mov   esi,CallTable
	jmp   int41hOK
NotZero:
	cmp   ebx,FunctionMax
	ja    int41hError
	shl   ebx,2
	add   ebx,CallTable
	cli
	call  dword[ebx]
	sti
	jc    int41hError
int41hOK:					       ; universal return procedure
	and	byte [esp+8],not 1		       ; clear CF
	iret

int41hError:					       ; universal return error procedure
	or	byte [esp+8],1			       ; set CF
	iret

 ;----------------------------------------------------;
 ;  TestInt                                           ;
 ;----------------------------------------------------;
TestInt:
	pushad
	push  es
	push  ds
 ;-----------------  
 ; some code here   
 ;-----------------       
	pop   ds
	pop   es      
	popad
	iret

 ;----------------------------------------------------;
 ;  unhandled_irq                                     ;
 ;----------------------------------------------------;
unhandled_irq:
	pushad
	push  es
	push  ds
	mov   byte [fs:0xB809C], "Q"
	mov   al,0x20
	out   0xa0,al
	out   0x20,al
	pop   ds
	pop   es
	popad
	iret

 ;----------------------------------------------------;
 ;  irq_clear                                         ;
 ;----------------------------------------------------;
irq_clear:
	push  eax
	mov   al,0x20
	out   0xa0,al
	out   0x20,al					; quiet screaming irq chip.
	pop   eax
	ret

 ;----------------------------------------------------;
 ;  Data.                                             ;
 ;----------------------------------------------------;
NumberOfTicks dd 0
Tick18	      db 0
testthis      dd 0
EsiPointer    dd 0
TimerProg     dd 0
TimerProgOn   db 0

