;=========================================================;
; Ata_Atapi16                                    11/12/03 ;
;---------------------------------------------------------;
; ATA/ATAPI functions.                                    ;
;                                                         ;
; (c) Craig Bamford, All rights reserved.                 ;
;=========================================================;

 ;----------------------------------------------------;
 ; GetAtaPiDrive      ; Gets the Ata/Atapi drive info ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;   Input:                                           ;
 ;          none.                                     ;
 ;  Output:                                           ;
 ;          Puts drive info in vals:                  ;
 ;                                             (100%) ;
 ;....................................................;

GetAtaPiDrive:
	mov   byte [Drive_num],0			
	mov   byte [command],0xec			
drive_id:
	mov   word [port],0x1f0 			
	mov   byte [drive],0x00 			
	call  AtaPi_Id_Drive				
	mov   word [port],0x1f0 			
	mov   byte [drive],0x10 			
	call  AtaPi_Id_Drive				
	mov   word [port],0x170 			
	mov   byte [drive],0x00 			
	call  AtaPi_Id_Drive				
	mov   word [port],0x170 			
	mov   byte [drive],0x10 			
	call  AtaPi_Id_Drive				
	cmp  byte[command],0xA1 			
	je   Lets_go			   
	mov  byte[command],0xA1 			
	jmp  drive_id
Lets_go:
	ret

;====================================================;
; AtaPi_Id_Drive                                     ;
;====================================================;

AtaPi_Id_Drive:
	mov   dx,0x7
	add   dx,word [port]
	mov   cx,0xffff
StatusReg1:				     
	in    al,dx
	and   al,0x80	 
	jz    WriteCommand1	    
	loop  StatusReg1
	jmp   DeviceBusy	     
WriteCommand1:				     
	mov   dx,0x6	     
	add   dx,word [port]
	mov   al,byte [drive]	    
	or    al,0xef
	out   dx,al
	mov   cx,0xffff
	mov   dx,0x7
	add   dx,word [port] 
StatusReg2:				     
	in    al,dx
	test  al,0x80		
	jz    Drdy_check		 
	loop  StatusReg2 
	jmp   DeviceBusy
Drdy_check:  
	test  al,0x40
	jnz   WriteCommand2
	cmp   byte[command],0xA1
	je    WriteCommand2 
DeviceBusy:	
	;stc
	ret

;====================================================;
;  WriteCommand2                                     ;
;====================================================;
WriteCommand2:
	mov   dx,0x7	       
	add   dx,word [port]
	mov   al,byte [command]      
	out   dx,al
	mov   cx,0xffff
	mov   dx,0x7
	add   dx,word [port] 
StatusReg3: 
	in    al,dx 
	test  al,0x80
	jnz   DrqErrorCheck1	     
	test  al,0x01
	jnz   error
	test  al,0x08 
	jnz   Read_data_reg_0	 
       ;stc
	ret

;====================================================;
; DrqErrorCheck                                      ;
;====================================================;
DrqErrorCheck1:
	push  cx
	mov   cx,0xFFFF
busy_delay123:
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	loop  busy_delay123
	pop   cx
	loop  StatusReg3
error:
       ;stc
	ret

;====================================================;
; Read data reg              (& move it into buffer) ;
;====================================================;
Read_data_reg_0:
	mov   dx,0	       
	add   dx,word [port]
	xor   ecx,ecx
	mov   cx,0x100
	mov   di,Temp_Buffer
Read_data_reg_1:
	in    ax,dx							   
	stosw
	loop  Read_data_reg_1		  
;====================================================;
; Label drive C                                      ;
;====================================================;
	cmp   byte[command],0xA1
	je    ItsAcd_1
	cmp   word[HdPort1],0
	jne   TryHdd2
	mov   dx,word [port]
	mov   word[HdPort1],dx
	mov   al,byte [drive]
	mov   byte[HdDrive1],al
	pusha
	mov   [LBAtest],0
	mov   di,Temp_Buffer
	mov   ax,word [di+2*49] 	      ; read capabilities field
	test  ax,0000001000000000b	      ; lba bit set?
	jz    .no_lba_drive_found	      ; either no drv or no lba
	mov   [LBAtest],1
.no_lba_drive_found:
	popa
	jmp   Byby
TryHdd2:
	cmp   word[HdPort2],0
	jne   Byby
	mov   dx,word [port]
	mov   word[HdPort2],dx
	mov   al,byte [drive]
	mov   byte[HdDrive2],al
	jmp   Byby
ItsAcd_1:
	cmp   word[CdPort1],0
	jne   TryCd2
	mov   dx,word [port]
	mov   word[CdPort1],dx
	mov   al,byte [drive]
	mov   byte[CdDrive1],al
	jmp   Byby
TryCd2: 
	cmp   word[CdPort2],0
	jne   Byby
	mov   dx,word [port]
	mov   word[CdPort2],dx
	mov   al,byte [drive]
	mov   byte[CdDrive2],al
	jmp   Byby
Byby:
	clc
	ret

;====================================================;
; Small Delay                                        ;
;====================================================;
Small_Delay:
	push  ecx
	mov   ecx,0x0ffff
BusyDelay3a:				      
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	loop  BusyDelay3a
	pop   ecx
	ret

;====================================================;
; Atapi Address                                      ;
;====================================================; 
Atapi_Address:
	mov   dx,word[CdPort1]
	mov   word[port],dx
	mov   al,byte[CdDrive1]
	mov   byte[drive],al 
	ret

;====================================================;
; Data.                                              ;
;====================================================;
command:	 db 0
port:		 dw 0
drive:		 db 0
AtapiError	 db 0
Drive_num:	 db 0

HdDrive1:	 db 0
HdPort1:	 dw 0
HdDrive2:	 db 0
HdPort2:	 dw 0

CdDrive1:	 db 0
CdPort1:	 dw 0
CdDrive2:	 db 0
CdPort2:	 dw 0
LBAtest 	 dw 0

Temp_Buffer rw	256
	      

